#ifndef SVN_CLIENT_HH
#define SVN_CLIENT_HH

#include <string>
#include <boost/function/function1.hpp>
#include <boost/function/function3.hpp>
#include <apr/time.hh>
#include <svn/pool.hh>
#include <svn_client.h>
#include <svn_types.h>

namespace svn {

class RevisionNumber;

namespace wc {
class Notification;
}

namespace client {

/** @see svn_client_prompt_t **/
typedef boost::function3<std::string,const std::string&,bool,Pool&> PromptFunc;

class RevisionSpecifier;

/** @see svn_client_auth_baton_t */
class AuthBaton {
        friend void checkout(boost::function1<void,const wc::Notification&>,
                AuthBaton, const std::string&, const std::string&,
                const RevisionSpecifier&, bool, Pool&);

        friend void update(AuthBaton, const std::string &path,
                const RevisionSpecifier &revision, bool recurse,
                boost::function1<void,const wc::Notification&> notifyFunction,
                Pool &pool);
    public:
        AuthBaton();
        void setPrompt(const PromptFunc &newPrompt) { prompt = newPrompt; }
        void setStoreAuthInfo(bool newVal) { me.store_auth_info = newVal; }
        bool getGotNewAuthInfo() const { return me.got_new_auth_info; }

    protected:
        struct svn_client_auth_baton_t me;

    private:
        static svn_error_t *prompt_callback(char **info, const char *prompt,
                svn_boolean_t hide, void *baton, apr_pool_t *pool);
        PromptFunc prompt;
};

/** @see svn_opt_revision_t **/
class RevisionSpecifier {
        friend void checkout(boost::function1<void,const wc::Notification&>,
                AuthBaton, const std::string&, const std::string&,
                const RevisionSpecifier&, bool, Pool&);

        friend void update(AuthBaton, const std::string &path,
                const RevisionSpecifier &revision, bool recurse,
                boost::function1<void,const wc::Notification&> notifyFunction,
                Pool &pool);
    public:
        const static RevisionSpecifier COMMITTED;
        const static RevisionSpecifier TRUNK;
        const static RevisionSpecifier PREVIOUS;
        const static RevisionSpecifier BASE;
        const static RevisionSpecifier WORKING;
        const static RevisionSpecifier HEAD;

        /** @see svn_opt_revision_kind **/
        enum Type {
            rUnspecified = svn_opt_revision_unspecified,
            rNumber = svn_opt_revision_number,
            rDate = svn_opt_revision_date,
            rCommitted = svn_opt_revision_committed,
            rPrevious = svn_opt_revision_previous,
            rBase = svn_opt_revision_base,
            rWorking = svn_opt_revision_working,
            rHead = svn_opt_revision_head,
        };

        RevisionSpecifier(const RevisionNumber&);
        RevisionSpecifier(const apr::Time&);

    protected:
        svn_opt_revision_t me;

    private:
        RevisionSpecifier(Type);
};

/**
 * Checks out a working copy of <tt>url</tt> at <tt>revision</tt>, using
 * <tt>path</tt> as the root directory of the newly-checked-out working
 * copy, and authenticating with <tt>authBaton</tt>.
 *
 * <p><tt>revision</tt> must be of kind <tt>rNumber</tt>, <tt>rHead</tt>,
 * or <tt>rDate</tt>. In the xml case, <tt>rUnspecified</tt> is also allowed.
 * If <tt>revision</tt> does not meet these requirements, throws a
 * <tt>SVN_ERR_CLIENT_BAD_REVISION</tt>.
 *
 * <p>Uses <tt>pool</tt> for any temporary allocation.
 **/
void checkout(boost::function1<void,const wc::Notification&> notifyFunction,
        AuthBaton, const std::string &url, const std::string &path,
        const RevisionSpecifier &revision, bool recurse, Pool &pool);

void update(AuthBaton, const std::string &path,
        const RevisionSpecifier &revision, bool recurse,
        boost::function1<void,const wc::Notification&> notifyFunction,
        Pool &pool);

void doSwitch(AuthBaton, const std::string &path, const std::string &url,
        const RevisionSpecifier &revision, bool recurse,
        boost::function1<void,const wc::Notification&>, Pool &pool);

void add(const std::string &path, bool recurse,
        boost::function1<void,const wc::Notification&>, Pool &pool);

}
}
#endif //!SVN_CLIENT_HH
